==> myri10ge-linux.1.2.0.tgz: myri10ge/linux/myri10ge_hal.h <==
#ifndef _MYRI10GE_HAL_H_
#define _MYRI10GE_HAL_H_

/* make sure linux/version.h is included here since it might be
 * removed from myri10ge.c one day */
#include <linux/version.h>

#include "myri10ge_version.h"
#include "myri10ge_checks.h"

#ifndef __iomem
#define __iomem
#endif

#ifndef __force
#define __force
#endif

#ifndef __unused
#define __unused __attribute__ ((unused))
#endif

#ifndef PCI_CAP_ID_VNDR
#define PCI_CAP_ID_VNDR 0x9
#endif

#ifndef PCI_CAP_ID_EXP
#define PCI_CAP_ID_EXP 0x10
#endif

#ifndef PCI_EXP_DEVCTL
#define PCI_EXP_DEVCTL 0x8
#endif

#ifndef PCI_EXP_DEVCTL_READRQ
#define PCI_EXP_DEVCTL_READRQ 0x7000
#endif

#ifndef PCI_EXP_LNKSTA
#define PCI_EXP_LNKSTA		18
#endif

#ifndef PCI_EXT_CAP_ID_ERR
#define PCI_EXT_CAP_ID_ERR	1
#endif

#ifndef PCI_ERR_CAP
#define PCI_ERR_CAP 24
#endif

#ifndef PCI_EXP_FLAGS_TYPE
#define PCI_EXP_FLAGS_TYPE 0x00f0
#endif

#ifndef PCI_EXP_TYPE_ROOT_PORT
#define PCI_EXP_TYPE_ROOT_PORT 0x4
#endif

#ifndef PCI_ERR_CAP_ECRC_GENC
#define PCI_ERR_CAP_ECRC_GENC	0x00000020
#endif

#ifndef PCI_ERR_CAP_ECRC_GENE
#define PCI_ERR_CAP_ECRC_GENE	0x00000040
#endif

#ifndef PCI_DEVICE_ID_NVIDIA_NFORCE_CK804_PCIE
#define PCI_DEVICE_ID_NVIDIA_NFORCE_CK804_PCIE 0x005d
#endif

#ifndef PCI_DEVICE_ID_NVIDIA_NFORCE_MCP55_PCIE_374
#define PCI_DEVICE_ID_NVIDIA_NFORCE_MCP55_PCIE_374 0x0374
#endif

#ifndef PCI_DEVICE_ID_NVIDIA_NFORCE_MCP55_PCIE_378
#define PCI_DEVICE_ID_NVIDIA_NFORCE_MCP55_PCIE_378 0x0378
#endif

/* the following PCI id is also defined in myri10ge.c
 * since it must be available in the kernel specific
 * and in this hal header. */
#ifndef PCI_DEVICE_ID_SERVERWORKS_HT2000_PCIE
#define PCI_DEVICE_ID_SERVERWORKS_HT2000_PCIE 0x0132
#endif

#ifndef PCI_DEVICE_ID_INTEL_E5000_PCIE23
#define PCI_DEVICE_ID_INTEL_E5000_PCIE23 0x25f7
#endif

#ifndef PCI_DEVICE_ID_INTEL_E5000_PCIE47
#define PCI_DEVICE_ID_INTEL_E5000_PCIE47 0x25fa
#endif

#ifndef PCI_VENDOR_ID_MYRICOM
#define PCI_VENDOR_ID_MYRICOM	0x14c1
#endif

#ifndef IRQF_SHARED
#define IRQF_SHARED SA_SHIRQ
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0) && !defined module_param
#define module_param(name, type, perm) MODULE_PARM(name, _MX_PARM_##type)
#define _MX_PARM_ulong "l"
#define _MX_PARM_charp "s"
#define _MX_PARM_int "i"
#else
#include <linux/moduleparam.h>
#endif

#ifdef CONFIG_64BIT
static inline void
myri10ge_pio_copy(void *to, void *from, size_t size)
{
	register volatile u64 *to64;
	volatile u64 *from64;
	size_t i;

	to64 = (volatile u64 *) to;
	from64 = from;
	for (i = (size / 8); i; i--) {
		__raw_writeq(*from64, to64);
		to64++;
		from64++;
	}
}
#else
static inline void
myri10ge_pio_copy(void __iomem *to, void *from, size_t size)
{
	memcpy_toio(to, from, size);
}
#endif

#ifndef MYRI10GE_HAVE_PM_MESSAGE_T
#define pm_message_t u32
#define pci_choose_state(pdev,pm) (pm)
#endif

#ifdef MYRI10GE_HAVE_PRIVATE_PM_STATE
#define pci_save_state(a) pci_save_state(a, mgp->pm_state)
#define pci_restore_state(a) pci_restore_state(a, mgp->pm_state)
#endif


/* Rather than using normal pci config space writes, we must map the
 * Nvidia config space ourselves.  This is because on opteron/nvidia
 * class machine the 0x{e,f}000000 mapping is handled by the nvidia
 * chipset, that means the internal PCI device (the on-chip
 * northbridge), or the amd-8131 bridge and things behind them are not
 * visible by this method.
 */

static u32 __iomem *
myri10ge_mmio_ext_config(struct pci_dev *dev, int where)
{
	u32 __iomem *ptr32 = NULL;
	static unsigned long config_phys = 0;
	unsigned long off, cfg_pa;
	u32 device, vendor;

	if (dev->vendor != PCI_VENDOR_ID_NVIDIA)
		return NULL;

	if (dev->device != PCI_DEVICE_ID_NVIDIA_NFORCE_CK804_PCIE &&
	    (dev->device < PCI_DEVICE_ID_NVIDIA_NFORCE_MCP55_PCIE_374 ||
	     dev->device > PCI_DEVICE_ID_NVIDIA_NFORCE_MCP55_PCIE_378))
		return NULL;

	if (config_phys == 0) {
		/* try to find the correct mapping by reading
		   the first word of config space, containing
		   the device and vendor ids */
		for (cfg_pa = 0xf0000000UL; cfg_pa >= 0xe0000000UL;
		     cfg_pa -= 0x10000000UL) {
			off = cfg_pa
				+ (unsigned long)dev->bus->number * 0x00100000UL
				+ (unsigned long)dev->devfn * 0x00001000UL;

			ptr32 = (u32 __iomem *) ioremap(off, 4);
			if (ptr32 == NULL) {
				continue;
			}
			vendor = *(u32 __force *)ptr32;
			device = vendor >> 16;
			vendor = vendor & 0xffff;
			iounmap(ptr32);
			if (vendor == dev->vendor &&
			    device == dev->device) {
				config_phys = cfg_pa;
				break;
			}
		}
	}
	if (config_phys == 0)
		return NULL;

	off = config_phys
		+ (unsigned long)dev->bus->number * 0x00100000UL
		+ (unsigned long)dev->devfn * 0x00001000UL
		+ where;
	ptr32 = (u32 __iomem *) ioremap(off, 4);
	return ptr32;
}

static int
myri10ge_read_ext_config_dword(struct pci_dev *dev, int where, u32 *val)
{
	u32 __iomem *ptr32;
	int status = pci_read_config_dword(dev, where, val);
	if (status && (ptr32 = myri10ge_mmio_ext_config(dev, where))) {
		*val = *(u32 __force *)ptr32;
		iounmap(ptr32);
		return 0;
	}
	return status;
}

static int
myri10ge_write_ext_config_dword(struct pci_dev *dev, int where, u32 val)
{
	u32 __iomem *ptr32;
	int status = pci_write_config_dword(dev, where, val);
	if (status &&  (ptr32 = myri10ge_mmio_ext_config(dev, where))) {
		*(u32 __force *) ptr32 = val;
		iounmap(ptr32);
		return 0;
	}
	return status;
}

#undef pci_find_ext_capability /* GPL only */
#define pci_find_ext_capability(dev, cap) 0

#ifndef NETIF_F_TSO
#define NETIF_F_TSO 0
#endif

#ifdef CONFIG_PCI_MSI
#define HAVE_MSI 1
#else
#define myri10ge_use_msi(mgp) 0
#endif

#ifndef	DMA_32BIT_MASK
#define DMA_32BIT_MASK ((dma_addr_t)0xffffffffULL)
#endif

#ifndef	DMA_64BIT_MASK
#define DMA_64BIT_MASK ((dma_addr_t)~0ULL)
#endif

/* do not static inline since gfp_t appeared recently */
#define myri10ge_kzalloc(size, flags)		\
({						\
	void *ret = kmalloc(size, flags);	\
	if (ret)				\
		memset(ret, 0, size);		\
	ret;					\
})

static inline void
myri10ge_msleep(unsigned int msecs)
{
	unsigned long timeout = HZ * msecs / 1000 + 1;
	while (timeout) {
		/* schedule_timeout_uninterruptible appeared in 2.6.15 */
		__set_current_state(TASK_UNINTERRUPTIBLE);
		timeout = schedule_timeout(timeout);
	}
}

#ifdef MYRI10GE_SKB_LINEARIZE_HAS_GFP
#define myri10ge_skb_linearize(skb) skb_linearize(skb, GFP_ATOMIC)
#else
#define myri10ge_skb_linearize skb_linearize
#endif

#ifdef NETIF_F_GSO_SHIFT
#define myri10ge_skb_is_gso skb_is_gso
#define MYRI10GE_GSO_SIZE gso_size
#else
#define myri10ge_skb_is_gso(skb) skb_shinfo(skb)->tso_size
#define MYRI10GE_GSO_SIZE tso_size
#endif

#ifdef MYRI10GE_SKB_PADTO_RETURNS_SKB
#define myri10ge_skb_padto(skb, len)	\
({					\
	skb = skb_padto(skb, ETH_ZLEN);	\
	skb == NULL;			\
})
#else /* MYRI10GE_SKB_PADTO_RETURNS_SKB */
#define myri10ge_skb_padto skb_padto
#endif /* MYRI10GE_SKB_PADTO_RETURNS_SKB */

#ifdef MYRI10GE_HAVE_NETDEV_ALLOC_SKB
#define myri10ge_netdev_alloc_skb netdev_alloc_skb
#else /* MYRI10GE_HAVE_NETDEV_ALLOC_SKB */
static inline struct sk_buff *
myri10ge_netdev_alloc_skb(struct net_device * dev, unsigned int length)
{
	struct sk_buff * skb = dev_alloc_skb(length);
	if (likely(skb))
		skb->dev = dev;
	return skb;
}
#endif /* MYRI10GE_HAVE_NETDEV_ALLOC_SKB */

/* CHECKSUM_HW replaced by CHECKSUM_PARTIAL and CHECKSUM_COMPLETE in 2.6.19 */
#ifndef CHECKSUM_PARTIAL
#define CHECKSUM_PARTIAL CHECKSUM_HW
#endif
#ifndef CHECKSUM_COMPLETE
#define CHECKSUM_COMPLETE CHECKSUM_HW
#endif

/* __GFP_COMP does not exist in some 2.6.5 kernels.
 * Just set it to 0 since we do not use it until 2.6.16 anyway */
#ifndef __GFP_COMP
#define __GFP_COMP 0
#endif

/* __be32 appeared with __bitwise in 2.6.9 */
#ifndef __bitwise
#define __bitwise
typedef u32 __be32;
#endif

#define MYRI10GE_ETHTOOL_OPS_TYPE typeof(*((struct net_device *)NULL)->ethtool_ops)

/* 2.6.20 splits skb->csum into an anonymous union csum (type __wsum) and csum_offset (__u32) */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,20)
#define MYRI10GE_SKB_CSUM_OFFSET csum_offset
#else
#define MYRI10GE_SKB_CSUM_OFFSET csum
typedef __u32 __bitwise __wsum;
typedef __u16 __bitwise __sum16;
static inline __wsum csum_unfold(__sum16 n) { return (__force __wsum)n; }
#endif

/* workstruct rework in 2.6.20,
 * work is passed has an argument to the function, mgp is its container */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,20)
#define MYRI10GE_INIT_WORK(work, func, arg) INIT_WORK(work, func)
#define MYRI10GE_WATCHDOG_ARG_TYPE struct work_struct *
#define MYRI10GE_WATCHDOG_ARG_CONTAINER_OF_MGP(ptr, type, member) container_of(ptr, type, member)
#else
#define MYRI10GE_INIT_WORK(work, func, arg) INIT_WORK(work, func, arg)
#define MYRI10GE_WATCHDOG_ARG_TYPE void *
#define MYRI10GE_WATCHDOG_ARG_CONTAINER_OF_MGP(ptr, type, member) ptr
#endif

#endif /* _MYRI10GE_HAL_H_ */

==> myri10ge-linux.1.3.0.tgz: myri10ge/linux/myri10ge_hal.h <==
#ifndef _MYRI10GE_HAL_H_
#define _MYRI10GE_HAL_H_

/* make sure linux/version.h is included here since it might be
 * removed from myri10ge.c one day */
#include <linux/version.h>

#include "myri10ge_version.h"
#include "myri10ge_checks.h"

#ifndef __iomem
#define __iomem
#endif

#ifndef __force
#define __force
#endif

#ifndef __unused
#define __unused __attribute__ ((unused))
#endif

#ifndef PCI_CAP_ID_VNDR
#define PCI_CAP_ID_VNDR 0x9
#endif

#ifndef PCI_CAP_ID_EXP
#define PCI_CAP_ID_EXP 0x10
#endif

#ifndef PCI_EXP_DEVCTL
#define PCI_EXP_DEVCTL 0x8
#endif

#ifndef PCI_EXP_DEVCTL_READRQ
#define PCI_EXP_DEVCTL_READRQ 0x7000
#endif

#ifndef PCI_EXP_LNKSTA
#define PCI_EXP_LNKSTA		18
#endif

#ifndef PCI_EXT_CAP_ID_ERR
#define PCI_EXT_CAP_ID_ERR	1
#endif

#ifndef PCI_ERR_CAP
#define PCI_ERR_CAP 24
#endif

#ifndef PCI_EXP_FLAGS_TYPE
#define PCI_EXP_FLAGS_TYPE 0x00f0
#endif

#ifndef PCI_EXP_TYPE_ROOT_PORT
#define PCI_EXP_TYPE_ROOT_PORT 0x4
#endif

#ifndef PCI_ERR_CAP_ECRC_GENC
#define PCI_ERR_CAP_ECRC_GENC	0x00000020
#endif

#ifndef PCI_ERR_CAP_ECRC_GENE
#define PCI_ERR_CAP_ECRC_GENE	0x00000040
#endif

#ifndef PCI_DEVICE_ID_NVIDIA_NFORCE_CK804_PCIE
#define PCI_DEVICE_ID_NVIDIA_NFORCE_CK804_PCIE 0x005d
#endif

#ifndef PCI_DEVICE_ID_NVIDIA_NFORCE_MCP55_PCIE_369
#define PCI_DEVICE_ID_NVIDIA_NFORCE_MCP55_PCIE_369 0x0369
#endif

#ifndef PCI_DEVICE_ID_NVIDIA_NFORCE_MCP55_PCIE_374
#define PCI_DEVICE_ID_NVIDIA_NFORCE_MCP55_PCIE_374 0x0374
#endif

#ifndef PCI_DEVICE_ID_NVIDIA_NFORCE_MCP55_PCIE_378
#define PCI_DEVICE_ID_NVIDIA_NFORCE_MCP55_PCIE_378 0x0378
#endif

/* the following PCI id is also defined in myri10ge.c
 * since it must be available in the kernel specific
 * and in this hal header. */
#ifndef PCI_DEVICE_ID_SERVERWORKS_HT2000_PCIE
#define PCI_DEVICE_ID_SERVERWORKS_HT2000_PCIE 0x0132
#endif

#ifndef PCI_DEVICE_ID_INTEL_E5000_PCIE23
#define PCI_DEVICE_ID_INTEL_E5000_PCIE23 0x25f7
#endif

#ifndef PCI_DEVICE_ID_INTEL_E5000_PCIE47
#define PCI_DEVICE_ID_INTEL_E5000_PCIE47 0x25fa
#endif

#ifndef PCI_VENDOR_ID_MYRICOM
#define PCI_VENDOR_ID_MYRICOM	0x14c1
#endif

#ifndef IRQF_SHARED
#define IRQF_SHARED SA_SHIRQ
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0) && !defined module_param
#define module_param(name, type, perm) MODULE_PARM(name, _MX_PARM_##type)
#define _MX_PARM_ulong "l"
#define _MX_PARM_charp "s"
#define _MX_PARM_int "i"
#else
#include <linux/moduleparam.h>
#endif

#ifdef CONFIG_64BIT
static inline void
myri10ge_pio_copy(void *to, void *from, size_t size)
{
	register volatile u64 *to64;
	volatile u64 *from64;
	size_t i;

	to64 = (volatile u64 *) to;
	from64 = from;
	for (i = (size / 8); i; i--) {
		__raw_writeq(*from64, to64);
		to64++;
		from64++;
	}
}
#else
static inline void
myri10ge_pio_copy(void __iomem *to, void *from, size_t size)
{
	memcpy_toio(to, from, size);
}
#endif

#ifndef MYRI10GE_HAVE_PM_MESSAGE_T
#define pm_message_t u32
#define pci_choose_state(pdev,pm) (pm)
#endif

#ifdef MYRI10GE_HAVE_PRIVATE_PM_STATE
#define pci_save_state(a) pci_save_state(a, mgp->pm_state)
#define pci_restore_state(a) pci_restore_state(a, mgp->pm_state)
#endif


/* Rather than using normal pci config space writes, we must map the
 * Nvidia config space ourselves.  This is because on opteron/nvidia
 * class machine the 0x{e,f}000000 mapping is handled by the nvidia
 * chipset, that means the internal PCI device (the on-chip
 * northbridge), or the amd-8131 bridge and things behind them are not
 * visible by this method.
 */

static u32 __iomem *
myri10ge_mmio_ext_config(struct pci_dev *dev, int where)
{
	u32 __iomem *ptr32 = NULL;
	struct pci_dev *mcp55;
	unsigned long base = 0UL;
	static unsigned long config_phys = 0;
	u32 pci_id;
	u16 word;
	unsigned long dev_base;

	if (config_phys == 0) {
		if (dev->vendor != PCI_VENDOR_ID_NVIDIA)
			goto unknown;

		if (dev->device == PCI_DEVICE_ID_NVIDIA_NFORCE_CK804_PCIE) {
			base = 0xe0000000UL;
		} else if (dev->device >= PCI_DEVICE_ID_NVIDIA_NFORCE_MCP55_PCIE_374 &&
			   dev->device <= PCI_DEVICE_ID_NVIDIA_NFORCE_MCP55_PCIE_378) {
			mcp55 = pci_find_slot(0, 0);
			if (mcp55 && mcp55->vendor == PCI_VENDOR_ID_NVIDIA &&
			    mcp55->device == PCI_DEVICE_ID_NVIDIA_NFORCE_MCP55_PCIE_369) {
				pci_read_config_word(mcp55, 0x90, &word);
				base = ((unsigned long)word & 0x7ffeU) << 25;
			}
		} else {
			goto unknown;
		}
		if (!base)
			return NULL;
		dev_base = (base 
			    + (unsigned long)dev->bus->number * 0x00100000UL
			    + (unsigned long)dev->devfn * 0x00001000UL);
		ptr32 = (uint32_t *) ioremap(dev_base, 4);
		if (!ptr32)
			return NULL;

		pci_id = *ptr32;
		iounmap(ptr32);
		if (pci_id != dev->vendor + (dev->device << 16)) {
			printk("Ext-conf-space at unknown address, contact help@myri.com\n");
			return NULL;
		}
		config_phys = base;
	}


	dev_base = (config_phys
		    + (unsigned long)dev->bus->number * 0x00100000UL
		    + (unsigned long)dev->devfn * 0x00001000UL);
	ptr32 = (u32 __iomem *) ioremap(dev_base + where, 4);
	return ptr32;

unknown:
	printk("Ext-conf-space with unknown chipset %x:%x\n",
	       dev->vendor, dev->device);
	printk("contact help@myri.com\n");
	return NULL;
}

static int
myri10ge_read_ext_config_dword(struct pci_dev *dev, int where, u32 *val)
{
	u32 __iomem *ptr32;
	int status = pci_read_config_dword(dev, where, val);
	if (status && (ptr32 = myri10ge_mmio_ext_config(dev, where))) {
		*val = *(u32 __force *)ptr32;
		iounmap(ptr32);
		return 0;
	}
	return status;
}

static int
myri10ge_write_ext_config_dword(struct pci_dev *dev, int where, u32 val)
{
	u32 __iomem *ptr32;
	int status = pci_write_config_dword(dev, where, val);
	if (status &&  (ptr32 = myri10ge_mmio_ext_config(dev, where))) {
		*(u32 __force *) ptr32 = val;
		iounmap(ptr32);
		return 0;
	}
	return status;
}

#undef pci_find_ext_capability /* GPL only */
#define pci_find_ext_capability(dev, cap) 0

#ifdef CONFIG_PCI_MSI
#define MYRI10GE_HAVE_MSI 1
#else
#define myri10ge_use_msi(mgp) 0
#endif

#ifndef	DMA_32BIT_MASK
#define DMA_32BIT_MASK ((dma_addr_t)0xffffffffULL)
#endif

#ifndef	DMA_64BIT_MASK
#define DMA_64BIT_MASK ((dma_addr_t)~0ULL)
#endif

/* do not static inline since gfp_t appeared recently */
#define myri10ge_kzalloc(size, flags)		\
({						\
	void *ret = kmalloc(size, flags);	\
	if (ret)				\
		memset(ret, 0, size);		\
	ret;					\
})

static inline void
myri10ge_msleep(unsigned int msecs)
{
	unsigned long timeout = HZ * msecs / 1000 + 1;
	while (timeout) {
		/* schedule_timeout_uninterruptible appeared in 2.6.15 */
		__set_current_state(TASK_UNINTERRUPTIBLE);
		timeout = schedule_timeout(timeout);
	}
}

#ifdef MYRI10GE_SKB_LINEARIZE_HAS_GFP
#define myri10ge_skb_linearize(skb) skb_linearize(skb, GFP_ATOMIC)
#else
#define myri10ge_skb_linearize skb_linearize
#endif

#ifdef NETIF_F_GSO_SHIFT
#define myri10ge_skb_is_gso skb_is_gso
#define MYRI10GE_GSO_SIZE gso_size
#else
#define myri10ge_skb_is_gso(skb) skb_shinfo(skb)->tso_size
#define MYRI10GE_GSO_SIZE tso_size
#endif

#ifdef MYRI10GE_SKB_PADTO_RETURNS_SKB
#define myri10ge_skb_padto(skb, len)	\
({					\
	skb = skb_padto(skb, ETH_ZLEN);	\
	skb == NULL;			\
})
#else /* MYRI10GE_SKB_PADTO_RETURNS_SKB */
#define myri10ge_skb_padto skb_padto
#endif /* MYRI10GE_SKB_PADTO_RETURNS_SKB */

#ifdef MYRI10GE_HAVE_NETDEV_ALLOC_SKB
#define myri10ge_netdev_alloc_skb netdev_alloc_skb
#else /* MYRI10GE_HAVE_NETDEV_ALLOC_SKB */
static inline struct sk_buff *
myri10ge_netdev_alloc_skb(struct net_device * dev, unsigned int length)
{
	struct sk_buff * skb = dev_alloc_skb(length);
	if (likely(skb))
		skb->dev = dev;
	return skb;
}
#endif /* MYRI10GE_HAVE_NETDEV_ALLOC_SKB */

/* CHECKSUM_HW replaced by CHECKSUM_PARTIAL and CHECKSUM_COMPLETE in 2.6.19 */
#ifndef CHECKSUM_PARTIAL
#define CHECKSUM_PARTIAL CHECKSUM_HW
#endif
#ifndef CHECKSUM_COMPLETE
#define CHECKSUM_COMPLETE CHECKSUM_HW
#endif

/* __GFP_COMP does not exist in some 2.6.5 kernels.
 * Just set it to 0 since we do not use it until 2.6.16 anyway */
#ifndef __GFP_COMP
#define __GFP_COMP 0
#endif

/* __be32 appeared with __bitwise in 2.6.9 */
#ifndef __bitwise
#define __bitwise
typedef u32 __be32;
#endif

#define MYRI10GE_ETHTOOL_OPS_TYPE typeof(*((struct net_device *)NULL)->ethtool_ops)

/* 2.6.20 splits skb->csum into an anonymous union csum (type __wsum) and csum_offset (__u32) */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,20)
#define MYRI10GE_SKB_CSUM_OFFSET csum_offset
#else
#define MYRI10GE_SKB_CSUM_OFFSET csum
typedef __u32 __bitwise __wsum;
typedef __u16 __bitwise __sum16;
static inline __wsum csum_unfold(__sum16 n) { return (__force __wsum)n; }
#endif

/* workstruct rework in 2.6.20,
 * work is passed has an argument to the function, mgp is its container */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,20)
#define MYRI10GE_INIT_WORK(work, func, arg) INIT_WORK(work, func)
#define MYRI10GE_WATCHDOG_ARG_TYPE struct work_struct *
#define MYRI10GE_WATCHDOG_ARG_CONTAINER_OF_MGP(ptr, type, member) container_of(ptr, type, member)
#else
#define MYRI10GE_INIT_WORK(work, func, arg) INIT_WORK(work, func, arg)
#define MYRI10GE_WATCHDOG_ARG_TYPE void *
#define MYRI10GE_WATCHDOG_ARG_CONTAINER_OF_MGP(ptr, type, member) ptr
#endif

/* myri10ge_tcp_v4_check lost its tcp header argument in 2.6.21 */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,21)
#define myri10ge_tcp_v4_check(th, len, saddr, daddr, base) tcp_v4_check(len, saddr, daddr, base)
#else
#define myri10ge_tcp_v4_check tcp_v4_check
#endif

#ifndef IA32_MSR_CR_PAT
#define IA32_MSR_CR_PAT 0x277
#endif

#if defined(CONFIG_X86) || defined (CONFIG_X86_64)
#define MYRI10GE_HAVE_PAT 1
#else
#define MYRI10GE_HAVE_PAT 0
#endif

#endif /* _MYRI10GE_HAL_H_ */
